"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makePaginateLazy = void 0;
const sequelize_1 = require("sequelize");
const utils_1 = require("./utils");
const LazyPaginationConnection_1 = __importDefault(require("./LazyPaginationConnection"));
function getLazyPaginationConnection(modelClass, paginateOptions, makePaginateOptions) {
    const primaryKeyField = makePaginateOptions?.primaryKeyField ?? (0, utils_1.getPrimaryKeyFields)(modelClass);
    const omitPrimaryKeyFromOrder = makePaginateOptions?.omitPrimaryKeyFromOrder ?? false;
    const { order: orderOption, where, after, before, limit, attributes, ...restQueryOptions } = paginateOptions;
    const normalizedOrder = (0, utils_1.normalizeOrder)(orderOption, primaryKeyField, omitPrimaryKeyFromOrder);
    const order = before ? (0, utils_1.reverseOrder)(normalizedOrder) : normalizedOrder;
    const cursor = after
        ? (0, utils_1.parseCursor)(after)
        : before
            ? (0, utils_1.parseCursor)(before)
            : null;
    const paginationQuery = cursor ? (0, utils_1.getPaginationQuery)(order, cursor) : null;
    const paginationWhere = paginationQuery
        ? { [sequelize_1.Op.and]: [paginationQuery, where] }
        : where;
    const paginationQueryOptions = {
        where: paginationWhere,
        limit,
        order,
        attributes,
        ...restQueryOptions,
    };
    const totalCountQueryOptions = {
        where,
        ...restQueryOptions,
    };
    const cursorCountQueryOptions = {
        where: paginationWhere,
        ...restQueryOptions,
    };
    return new LazyPaginationConnection_1.default({
        getEdgesPromise: async () => {
            const instances = await modelClass.findAll(paginationQueryOptions);
            if (before) {
                instances.reverse();
            }
            return instances.map((node) => ({
                node,
                cursor: (0, utils_1.createCursor)(node, order),
            }));
        },
        getTotalCountPromise: () => (0, utils_1.getCount)(modelClass, totalCountQueryOptions),
        getCursorCountPromise: () => (0, utils_1.getCount)(modelClass, cursorCountQueryOptions),
        isBefore: Boolean(before),
    });
}
const makePaginate = (model, makePaginateOptions) => {
    async function paginate(paginateOptions) {
        const modelClass = (0, utils_1.isModelClass)(this)
            ? this
            : model;
        const connection = getLazyPaginationConnection(modelClass, paginateOptions, makePaginateOptions);
        const [edges, totalCount, pageInfo] = await Promise.all([
            connection.getEdges(),
            connection.getTotalCount(),
            connection.getPageInfo(),
        ]);
        return {
            totalCount,
            edges,
            pageInfo,
        };
    }
    return paginate;
};
function makePaginateLazy(model, makePaginateOptions) {
    function paginateLazy(paginateOptions) {
        const modelClass = (0, utils_1.isModelClass)(this)
            ? this
            : model;
        return getLazyPaginationConnection(modelClass, paginateOptions, makePaginateOptions);
    }
    return paginateLazy;
}
exports.makePaginateLazy = makePaginateLazy;
exports.default = makePaginate;
